--- 战斗技巧脚本
local A = { ["flag"] = Anchor().CENTER, ["left"] = -681, ["top"] = -212, ["width"] = 250, ["height"] = 30 };
local B = { ["flag"] = Anchor().CENTER, ["left"] = -404, ["top"] = -212, ["width"] = 250, ["height"] = 30 };
local C = { ["flag"] = Anchor().CENTER, ["left"] = -126, ["top"] = -212, ["width"] = 250, ["height"] = 30 };
local D = { ["flag"] = Anchor().CENTER, ["left"] = 152, ["top"] = -212, ["width"] = 250, ["height"] = 30 };
local E = { ["flag"] = Anchor().CENTER, ["left"] = 433, ["top"] = -212, ["width"] = 250, ["height"] = 30 };

local F = { ["flag"] = Anchor().CENTER, ["left"] = -681, ["top"] = 381, ["width"] = 250, ["height"] = 30 };
local G = { ["flag"] = Anchor().CENTER, ["left"] = -404, ["top"] = 381, ["width"] = 250, ["height"] = 30 };
local H = { ["flag"] = Anchor().CENTER, ["left"] = -126, ["top"] = 381, ["width"] = 250, ["height"] = 30 };
local I = { ["flag"] = Anchor().CENTER, ["left"] = 152, ["top"] = 381, ["width"] = 250, ["height"] = 30 };
local J = { ["flag"] = Anchor().CENTER, ["left"] = 433, ["top"] = 381, ["width"] = 250, ["height"] = 30 };

local DETECT_HERO_ROLE_RECT_INFO = { A, B, C, D, E, F, G, H, I, J };

local FLOAT_WINDOW_POS_WHEN_LOADING = {
    anchor = Anchor().CENTER,
    x = 796,
    y = -384
}

local BATTLE_SKILL_VIEW_POS_WHEN_LOADING = {
    x = 504,
    y = 212
};

local BATTLE_SKILL_VIEW_POS_WHEN_RUNNING = {
    x = 500;
    y = 100;
};


--- 英雄类型
local HERO_TYPE_ENEMY = 1;
local HERO_TYPE_SELF = 2;

-- 一局游戏中时候已经做过英雄角色的检测
local hasDoDetectHeroRole = false;
local cbBattleSkill = nil;
local isCbBattleSkillChecked = true;

local battleSkillView = nil;
local floatBall = nil;
local settingView = nil;

--- 战斗技巧界面
local hasSetHeroInfo = false;
local enemyHeroIdList = nil;
local currentHeroId = nil;

local isShowingWhenFullCover = false;

------- 打印Log --------
local LOG = "battle_skill";
local function printLog(msg)
    commonlib_print(LOG, msg);
end

local function moveFloatWindowTo(position)
    commonlib_runOnUIThread(function()
        floatBall:moveCenterTo(position.anchor, position.x, position.y);
    end)
end

------- 界面 View 相关
local function setBattleSkillViewListener()
    battleSkillView:setOnCloseBtnClickListener(function()
        printLog("battleSkillView 关闭按钮被点击");
        battleSkillView:hide();

        local currentState = commonlib_getGameParam(KEY_GAME_STATE);
        if currentState == GAME_STATE_LOADING then
            moveFloatWindowTo(FLOAT_WINDOW_POS_WHEN_LOADING);
        end

        if currentState == GAME_STATE_LOADING or currentState == GAME_STATE_RUNNING then
            floatBall:show();
        end
    end);

    battleSkillView:setOnSettingBtnClickListener(function()
        printLog("battleSkillView 设置按钮被点击 ");
        battleSkillView:hide();
        settingView:show();
    end);
end

local function setFloatBallListener()
    floatBall:setOnClickListener(function()
        printLog("悬浮球被点击：isCbBattleSkillChecked ： " .. tostring(isCbBattleSkillChecked));

        if isCbBattleSkillChecked and hasSetHeroInfo then
            printLog("显示 battleSkillView");
            battleSkillView:show();
        else
            printLog("显示 settingView");
            settingView:show();
        end
        floatBall:hide();
    end);
end

local function setSettingViewListener()
    settingView:setOnEventListener(function()
    end, function()
        printLog("设置界面被隐藏");
        if isCbBattleSkillChecked and hasSetHeroInfo then
            battleSkillView:show();
        else
            floatBall:show();
        end
    end);
end

local function getViews()
    battleSkillView = ClickableFloatView():addBattleSkillView(Anchor().TOPLEFT, BATTLE_SKILL_VIEW_POS_WHEN_RUNNING.x, BATTLE_SKILL_VIEW_POS_WHEN_RUNNING.y, 3);
    settingView = SettingView();
    floatBall = TopSmallView();

    battleSkillView:hide();
    settingView:hide();
    floatBall:hide();

    printLog("battleSkillView : " .. tostring(battleSkillView));
    printLog("floatBall : " .. tostring(floatBall));
    printLog("settingView : " .. tostring(settingView));
end

local function setViewListener()
    setFloatBallListener();
    setBattleSkillViewListener();
    setSettingViewListener();
end

---  判断英雄个数是否有效
local function isHeroInfoValid(currentHeroId, heroIdTable)
    if currentHeroId == nil then
        printLog("未获取到当前英雄的用户角色");
        return false;
    end

    if heroIdTable == nil then
        printLog("未获取到敌方英雄的用户角色");
        return false;
    end

    if #heroIdTable == 0 then
        printLog("敌方英雄的个数为0个");
        return false;
    end

    return true;
end

local DEFAULT_BATTLE_SKILL_TEXT = "超人小编正在努力研究中，点击其他敌方英雄看看吧！";
local HERO_AVATOR_IMG_PATH_PREFIX = "lua/img/hero_avator/";
local HERO_AVATOR_IMG_PATH_SUFFIX = ".png";
local function setBattleSkillViewHeroInfo(currentHeroId, heroIdTable)
    if not isHeroInfoValid(currentHeroId, heroIdTable) then
        printLog("英雄信息无效，不应该作展示");
        return;
    end

    commonlib_runOnUIThread(function()
        printLog("开始设置 BattleSkillView 控件的敌方英雄信息");
        local skillInfoMap = {};

        print("HERO_COUNTER_INFO_TABLE 是否为空：" .. tostring(HERO_COUNTER_INFO_TABLE == nil));

        for index, heroId in ipairs(heroIdTable) do
            local heroAvatorImgPath = HERO_AVATOR_IMG_PATH_PREFIX .. tostring(heroId) .. HERO_AVATOR_IMG_PATH_SUFFIX;
            local heroBattleSkillTextKey = tostring(currentHeroId) .. " , " .. tostring(heroId);
            local heroBattleSkillText = HERO_COUNTER_INFO_TABLE[heroBattleSkillTextKey];

            if heroBattleSkillText == nil or string.len(heroBattleSkillText) == 0 then
                printLog("文本库中没有对应文本，选择默认文本");
                heroBattleSkillText = DEFAULT_BATTLE_SKILL_TEXT;
            end

            local skillInfo = {};
            skillInfo["heroAvatorImgPath"] = heroAvatorImgPath;
            skillInfo["heroBattleSkillText"] = heroBattleSkillText;

            skillInfoMap[#skillInfoMap + 1] = skillInfo;
        end

        battleSkillView:setBattleSkillInfo(skillInfoMap);
        hasSetHeroInfo = true;
        printLog("设置敌方英雄信息成功");

        if commonlib_getGameParam(KEY_GAME_STATE) == GAME_STATE_LOADING then
            commonlib_runOnUIThread(function()
                battleSkillView:setLoadingUI();
                battleSkillView:show();
                battleSkillView:moveTo(BATTLE_SKILL_VIEW_POS_WHEN_LOADING.x, BATTLE_SKILL_VIEW_POS_WHEN_LOADING.y);
            end);
        end
    end);
end

local function initViews()

    getViews();

    setViewListener();

end

local KEY_IS_CHECKBOX_SELECTED = "isCheckboxSelected";
------- 设置版面UI -------
local function onBattleSkillCheckboxSelected(checkboxId, isChecked)
    if checkboxId == cbBattleSkill:getId() then
        isCbBattleSkillChecked = isChecked;
    end

    commonlib_setSharedPrefBoolean(KEY_IS_CHECKBOX_SELECTED, isChecked);
end

local BATTLE_SKILL_TAB_PAGE_NAME = "战斗宝典";
local BATTLE_SKILL_CHECKBOX_TEXT = "开启对战技巧";
local function initBattleSkillUI()
    printLog("battle skill initConfigUI");
    local tabPage = settingView:addTableView(BATTLE_SKILL_TAB_PAGE_NAME);

    local Param;
    local cbParamProperty;

    Param = create("RelativeLayoutParam");
    cbParamProperty = {
        layout_marginTop = "20dp",
        layout_marginLeft = "25dp"
    };
    Param:setLayoutParam(cbParamProperty);
    cbBattleSkill = tabPage:addCheckBoxView(BATTLE_SKILL_CHECKBOX_TEXT, true, Param, onBattleSkillCheckboxSelected);

    cbBattleSkill:setChecked(commonlib_getSharedPrefBoolean(KEY_IS_CHECKBOX_SELECTED, true));
end

------ 检测英雄角色相关 ------
local function shouldDoDetect()
    if not isCbBattleSkillChecked then
        printLog("总Checkbox未选中，不做英雄角色识别");
        return false;
    end

    if hasDoDetectHeroRole then
        printLog("已经做过英雄角色的识别，不在进行第二次识别");
        return false;
    end

    if enemyHeroIdList ~= nil then
        printLog("已经识别到英雄角色，不再做识别");
        return false;
    end

    printLog("应该做英雄角色的识别");

    return true;
end

local function printHeroInfoTable(heroInfoTable)
    printLog("heroInfoTable is null: " .. tostring(heroInfoTable == nil));
    printLog("heroInfoTable size:" .. tostring(#heroInfoTable));

    for index, heroInfo in ipairs(heroInfoTable) do
        printLog("print hero index : " .. tostring(index));

        printLog("hero type : " .. tostring(heroInfo["type"]));
        printLog("hero id : " .. tostring(heroInfo["id"]));

        printLog("hero name : " .. tostring(HERO_ID_TABLE[tostring(heroInfo["id"])]));
    end
end

local function parseHeroInfo(heroInfoTable)
    printLog("开始解析英雄数据");
    currentHeroId = nil;
    enemyHeroIdList = {};

    for index, heroInfo in ipairs(heroInfoTable) do
        local heroType = heroInfo["type"];
        local heroId = heroInfo["id"];

        if heroId ~= -1 then
            if heroType == HERO_TYPE_SELF then
                currentHeroId = heroId;
            elseif heroType == HERO_TYPE_ENEMY then
                enemyHeroIdList[#enemyHeroIdList + 1] = heroId;
            end
        end
    end

    if currentHeroId == nil or enemyHeroIdList == nil or #enemyHeroIdList == 0 then
        printLog("未获取到己方英雄信息或敌方英雄个数为0个");
        currentHeroId = nil;
        enemyHeroIdList = nil;
    end
end

local function handleDectHeroRole(heroInfoTable)
    printLog("处理英雄识别结果");
    printLog("识别到的个数: " .. tostring(#heroInfoTable));
    printLog("结果是否为空：" .. tostring(heroInfoTable == nil));

    if heroInfoTable == nil then
        printLog("接口返回的值为nil");
        return;
    else
        printLog("接口返回的值不为nil");
    end

    printHeroInfoTable(heroInfoTable);
    parseHeroInfo(heroInfoTable);
    setBattleSkillViewHeroInfo(currentHeroId, enemyHeroIdList);
end

local function onDetectHeroRoleCallback(heroInfoTable)
    printLog("英雄识别接口回调");
    handleDectHeroRole(heroInfoTable);
end

-------- 游戏状态回调 ------
local function onGameLoading()
    printLog("onGameLoading .....")
    if shouldDoDetect() then
        hasDoDetectHeroRole = true;
        printLog("开始进行英雄角色识别");

        --- 采用回调方式
        --commonlib_detectHeroRole(DETECT_HERO_ROLE_RECT_INFO, onDetectHeroRoleCallback)

        --- 不采用回调方式
        local beginTime = commonlib_getCurrentTime();
        local heroInfoTable = commonlib_detectHeroRole(DETECT_HERO_ROLE_RECT_INFO);
        local endTime = commonlib_getCurrentTime();
        printLog("检测英雄角色的执行时间：" .. tostring(endTime - beginTime));

        handleDectHeroRole(heroInfoTable);
    end
end

local function onJustEnterGame()
    printLog("onJustEnterGame ...... ");

    commonlib_runOnUIThread(function()
        battleSkillView:setRunningUI();
        battleSkillView:moveTo(BATTLE_SKILL_VIEW_POS_WHEN_RUNNING.x, BATTLE_SKILL_VIEW_POS_WHEN_RUNNING.y);
        battleSkillView:hide();
    end);
end

local function onFullCoverStateChange(isFullCover)
    printLog("onFullCoverStateChange......." .. tostring(isFullCover));
    if isFullCover then
        commonlib_runOnUIThread(function()
            isShowingWhenFullCover = battleSkillView:isShowing();
            printLog("全遮挡发生时是否正在显示：" .. tostring(isShowingWhenFullCover));
            battleSkillView:hide();
        end);
    else
        if isShowingWhenFullCover then
            commonlib_runOnUIThread(function()
                battleSkillView:show();
            end);
        end
    end
end

local function onGameOver()
    printLog("onGameOver......");

    commonlib_runOnUIThread(function()
        battleSkillView:hide();
        settingView:hide();
        floatBall:hide();
    end);
    currentHeroId = nil;
    enemyHeroIdList = nil;
    hasDoDetectHeroRole = false;
    hasSetHeroInfo = false;
end

local function registerGameEvents()
    printLog("注册游戏事件");

    commonlib_registerCallback(common_game_events_callbacks.common_game_loading, onGameLoading);

    commonlib_registerCallback(common_game_events_callbacks.common_just_enter_game, onJustEnterGame);

    commonlib_registerCallback(common_game_events_callbacks.common_full_cover, onFullCoverStateChange)

    commonlib_registerCallback(common_game_events_callbacks.common_game_over, onGameOver);
end

local function unregisterGameEvents()

    commonlib_unregisterCallback(common_game_events_callbacks.common_game_loading, onGameLoading);

    commonlib_unregisterCallback(common_game_events_callbacks.common_just_enter_game, onJustEnterGame);

    commonlib_unregisterCallback(common_game_events_callbacks.common_full_cover, onFullCoverStateChange)

    commonlib_unregisterCallback(common_game_events_callbacks.common_game_over, onGameOver);
end

local function init()
    printLog("battle_skill init");

    initViews();

    initBattleSkillUI();

    registerGameEvents();

    printLog("battle_skill init finish ");
end

local function uninit()
    unregisterGameEvents();
end

table.insert(common_life_circle_callbacks, { init, uninit });