-- 公共库脚本
--[[
-- 该脚本是为其他脚本提供公共的变量，函数的库脚本，类似于java的工具类
-- 该脚本里面的变量和函数都是全局可见的，在Lua里面先加载，其他各个脚本
-- 直接通过变量名或者函数名引用
]]

--- 上报示例
--- 1.report = Report();对应的类为ReportLib
--- ["report_data"]这个key对应的LuaTable中可以传入多个上报字段
--- 2.report:report({"table_name" = "xxx", "report_data" = {"skin" = 1, "battle" = 1}});

--- ******游戏运行状态相关******----
KEY_GAME_STATE = "gameState";
GAME_STATE_NOTSTART = -1; -- 游戏未开始
GAME_STATE_LOADING = 0; -- 游戏加载状态
GAME_STATE_RUNNING = 1; -- 游戏运行状态
GAME_STATE_FINISH = 2; -- 游戏结束状态

KEY_IS_JUST_ENTER_GAME = "isJustEnterGame"; -- 是否刚刚进入游戏

--- ******界面元素******----
------------- 界面不可变元素-------------
KEY_IS_IN_5V5_MAP = "isIn5v5Map"; -- 是否是在5v5王者峡谷地图

KEY_HAS_PAUSE_BUTTON = "hasPauseButton"; -- 是否有暂停按钮
KEY_HAS_SETTING_BUTTON = "hasSettingButton"; -- 是否有设置按钮

KEY_SUMMONER_SKILL_COUNT = "summonerSkillCount"; -- 召唤师技能个数
SUMMONER_SKILL_COUNT_TWO = 2; -- 两个召唤师技能，新玩家在召唤师峡谷中只有两个：恢复和回城
SUMMONER_SKILL_COUNT_THREE = 3; -- 三个召唤师技能：恢复、回城和自选技能
SUMMONER_SKILL_COUNT_FOUR = 4; -- 在迷雾四个召唤师技能：监视、恢复、回城和自选技能

KEY_HERO_SKILL_COUNT = "heroSkillCount"; -- 英雄技能个数
HERO_SKILL_COUNT_THREE = 3; -- 英雄有三个技能，
HERO_SKILL_COUNT_FOUR = 4; -- 英雄有四个技能，这类英雄有小乔和干将莫邪

------------ 界面可变元素，在每一次打开设置窗口时需要进行识别------------
KEY_SHOP_MENU_POSITION = "shopMenuPosition"; -- 商店菜单位置
SHOP_MENU_POSITION_LEFT = "LEFT"; -- 商店菜单位于左侧
SHOP_MENU_POSITION_RIGHT = "RIGHT"; -- 商店菜单位于右侧

KEY_HERO_ATTACK_MODE = "heroAttackMode"; -- 英雄攻击模式
HERO_ATTACK_MODE_FREE = "FREE"; -- 自由攻击模式，默认的攻击默认
HERO_ATTACK_MODE_ADVANCED_FREE = "ADVANCED_FREE"; -- 高级自由攻击模式，即推塔补刀模式
HERO_ATTACK_MODE_LOCK = "LOCK"; -- 锁定攻击模式，即英雄小兵模式

--- ******屏幕遮挡相关******----
KEY_IS_FULL_COVER = "isFullCover"; -- 屏幕是否全遮挡

--- 半遮挡相关
KEY_IS_CHAT_WINDOW_OPEN = "isChatWindowOpen"; -- 聊天窗口是否打开

KEY_LARGE_MAP = "largeMap";
LARGE_MAP_HIDE = "HIDE"; -- 默认状态，此时小地图位于左侧且不展开
LARGE_MAP_LEFT_ZOOMOUT = "LEFT_ZOOMOUT"; -- 左侧展开状态
LARGE_MAP_RIGHT_ZOOMOUT = "RIGHT_ZOOMOUT"; -- 右侧展开状态

--游戏状态或者内容事件的key
COMMON_GAME_LOADING_KEY = "common_game_loading"
COMMON_GAME_JUST_ENTER_KEY = "common_just_enter_game"
COMMON_GAME_START_KEY = "common_game_start"
COMMON_GAME_RUNNING_KEY = "common_game_running"
COMMON_GAME_OVER_KEY = "common_game_over"
COMMON_FULL_COVER_KEY = "common_full_cover"
COMMON_MAP_MODE_KEY = "common_map_mode"
COMMON_CHAT_WINDOW_KEY = "common_chat_window"
COMMON_EQUIPMENT_KEY = "common_equipment"
COMMON_ATTACK_MODE_KEY = "common_attck_mode"

X_OFFSET_IN_18TO9_SCREEN = - 117;

-- 用于调用到各个业务脚本的生命周期函数表
common_life_circle_callbacks = {}

-- 用于回调到各个业务脚本的回调函数表
common_game_events_callbacks = {
    [COMMON_GAME_LOADING_KEY] = {},
    [COMMON_GAME_JUST_ENTER_KEY] = {},
    [COMMON_GAME_START_KEY] = {},
    [COMMON_GAME_RUNNING_KEY] = {},
    [COMMON_GAME_OVER_KEY] = {},
    [COMMON_FULL_COVER_KEY] = {},
    [COMMON_MAP_MODE_KEY] = {},
    [COMMON_CHAT_WINDOW_KEY] = {},
    [COMMON_EQUIPMENT_KEY] = {},
    [COMMON_ATTACK_MODE_KEY] = {}
}
core_scripts = {}

local TAG = "common_lib"
local imageRecognize = ImageRecognize()
local gameConfig = GlobalGameConfig()
local sharedPrefConfig = SharedPrefConfig()
local coordinateTool = CoordinateTool()

-- 支持添加tag的打印log函数
function commonlib_print(tag, msg)
    if tag == nil or msg == nil then
        print("tag or msg is nil")
        return
    end
    print("[" .. tag .. "]" .. tostring(msg))
end

-- 判断表是否为空的函数
function commonlib_isTableEmpty(table)
    if table == nil or #table == 0 then
        return true
    end
    return false
end

-- 获取当前时间的函数
function commonlib_getCurrentTime()
    local system = create("System")
    if system ~= nil then
        return system:currentTimeMillis()
    end
    commonlib_print(TAG, "commonlib_getCurrentTime system is nil")
    return 0
end

-- 判断是否发生了全遮挡或者地图展开
function commonlib_isFullCoverOrMapExpand(fullCover, mapMode)
    return fullCover or mapMode == LARGE_MAP_LEFT_ZOOMOUT or mapMode == LARGE_MAP_RIGHT_ZOOMOUT
end

-- 获取游戏的配置参数函数
function commonlib_getGameParam(key)
    if gameConfig ~= nil and key ~= nil then
        return gameConfig:get(key)
    end
    commonlib_print(TAG, "commonlib_getGameConfig params is nil")
    return nil
end

-- 设置游戏的配置参数函数
function commonlib_setGameParam(key, value)
    if gameConfig ~= nil and key ~= nil and value ~= nil then
        commonlib_print(TAG, "commonlib_setGameParam: " .. tostring(key) .. " = " .. tostring(value))
        gameConfig:set(key, value)
    else
        commonlib_print(TAG, "commonlib_setGameParam params is nil")
    end
end

-- 获取游戏偏好设置参数函数
function commonlib_getSharedPrefString(key, def)
    if sharedPrefConfig ~= nil and key ~= nil and def ~= nil then
        return sharedPrefConfig:getString(key, def)
    end
    commonlib_print(TAG, "commonlib_getSharedPrefString params is nil")
    return nil
end

-- 设置游戏偏好参数函数
function commonlib_setSharedPrefString(key, value)
    if sharedPrefConfig ~= nil and key ~= nil and value ~= nil then
        --commonlib_print(TAG, "commonlib_setSharedPrefString: " .. tostring(key) .. " = " .. tostring(value))
        sharedPrefConfig:setString(key, value)
    else
        commonlib_print(TAG, "commonlib_setSharedPrefString params is nil")
    end
end

function commonlib_getSharedPrefBoolean(key, def)
    if sharedPrefConfig ~= nil and key ~= nil and def ~= nil then
        return sharedPrefConfig:getBoolean(key, def)
    end
    commonlib_print(TAG, "commonlib_getSharedPrefBoolean params is nil")
    return nil
end

function commonlib_setSharedPrefBoolean(key, value)
    if sharedPrefConfig ~= nil and key ~= nil and value ~= nil then
        --commonlib_print(TAG, "commonlib_setSharedPrefString: " .. tostring(key) .. " = " .. tostring(value))
        sharedPrefConfig:setBoolean(key, value)
    else
        commonlib_print(TAG, "commonlib_setSharedPrefBoolean params is nil")
    end
end

function commonlib_isScreenRatio18To9()
    if coordinateTool ~= nil then
        return coordinateTool:isScreenRatio18To9();
    else
        commonlib_print(TAG, "commonlib_isScreenRatio18To9 params is nil")
        return false;
    end
end

-- 在UI线程调用的函数
function commonlib_runOnUIThread(func)
    local uiThread = UiThread()
    if uiThread ~= nil and func ~= nil then
        uiThread:run(func)
    else
        commonlib_print(TAG, "commonlib_runOnUIThread uiThread or func is nil")
    end
end

-- 注册回调函数的方法
function commonlib_registerCallback(tableName, funcName)
    if tableName ~= nil and funcName ~= nil then
        table.insert(tableName, funcName)
    else
        commonlib_print(TAG, "commonlib_registerCallback tableName or funcName is nil")
    end
end

-- 反注册回调函数的方法
function commonlib_unregisterCallback(tableName, funcName)
    if tableName ~= nil and funcName ~= nil then
        for i = #tableName, 1, -1 do
            if tableName[i] == funcName then
                table.remove(tableName, i)
            end
        end
    else
        commonlib_print(TAG, "commonlib_unregisterCallback tableName or funcName is nil")
    end
end

-- 图像识别相关的函数
-- 添加识别参照系图image
function commonlib_recognizeAddImage(imagePath)
    if imageRecognize ~= nil and imagePath ~= nil then
        imageRecognize:addImage(imagePath)
    else
        commonlib_print(TAG, "commonlib_recognizeAddImage imageRecognize or imagePath is nil")
    end
end

-- 根据参照图检测匹配度，识别图像是否存在
function commonlib_recognizeMatchResource(rect, path)
    if imageRecognize ~= nil and rect ~= nil and path ~= nil then
        return imageRecognize:matchResource(rect, path)
    else
        commonlib_print(TAG, "commonlib_recognizeMatchResource imageRecognize or rect or path is nil")
        return nil
    end
end

-- 用于判断红蓝buffer
function commonlib_recognizeMatchRedBlueBuffer(rect)
    if imageRecognize ~= nil and rect ~= nil then
        return imageRecognize:matchRedBlueBuffer(rect)
    else
        commonlib_print(TAG, "commonlib_recognizeMatchRedBlueBuffer imageRecognize or rect is nil")
        return nil
    end
end

-- 根据匹配的小地图位置检测出英雄数组
function commonlib_recognizeFindObjectPosition(rect)
    if imageRecognize ~= nil and rect ~= nil then
        return imageRecognize:findObjectPosition(rect)
    else
        commonlib_print(TAG, "commonlib_recognizeFindObjectPosition imageRecognize or rect is nil")
        return nil
    end
end

--function commonlib_detectHeroRole(rect, callback)
--    if imageRecognize ~= nil and rect ~= nil and callback ~= nil then
--        imageRecognize:detectHeroRole(rect, callback)
--    else
--        commonlib_print(TAG, "commonlib_detectHeroRole imageRecognize or rect is nil")
--    end
--end
--- 识别英雄角色
function commonlib_detectHeroRole(rect)
    if imageRecognize ~= nil and rect ~= nil then
        return imageRecognize:detectHeroRole(rect)
    else
        commonlib_print(TAG, "commonlib_detectHeroRole imageRecognize or rect is nil")
        return nil
    end
end