--- 显示悬浮窗按钮的脚本

local floatWindow
local lastShopMenuPos

local FLOAT_TAG = "float_window"
local GAMEMASTER_GAME_ASSISTANT_SRV = "gamemaster_game_assistant_srv"
local FLOAT_WINDON_SHOW = 1;
local FLOAT_WINDON_NOT_SHOW = 2;
local hasReportFloatWindowData = false;
local FLOAT_WINDOW_POS_WHEN_SHOP_MENU_POSITION_LEFT = {
    anchor = Anchor().TOPRIGHT,
    x = -172,
    y = 153
}

local FLOAT_WINDOW_POS_WHEN_SHOP_MENU_POSITION_RIGHT = {
    anchor = Anchor().TOPLEFT,
    x = 340,
    y = 421
}

local function printLog(msg)
    commonlib_print(FLOAT_TAG, msg)
end

local function reportFloatEntranceShowState(state)
    printLog("float_window===================" .. "埋点测试" .. tostring(state));
    local reportLib = Report();
    if reportLib ~= nil then
        printLog("float_window===================" .. "埋点测试");
        reportLib:report({
            ["table_name"] = GAMEMASTER_GAME_ASSISTANT_SRV,
            ["report_data"] = { ["show_float"] = state }
        })
    end
end

local function moveFloatWindowTo(position)
    commonlib_runOnUIThread(function()
        floatWindow:moveCenterTo(position.anchor, position.x, position.y);
    end)
end

-- 根据商店菜单的位置移动悬浮窗
local function moveFloatWindow(shopMenuPosition)
    if shopMenuPosition == SHOP_MENU_POSITION_LEFT then
        moveFloatWindowTo(FLOAT_WINDOW_POS_WHEN_SHOP_MENU_POSITION_LEFT);
    else
        moveFloatWindowTo(FLOAT_WINDOW_POS_WHEN_SHOP_MENU_POSITION_RIGHT);
    end
end

local function showFloatWindow()
    printLog("showFloatWindow")
    commonlib_runOnUIThread(function()
        if not floatWindow:isShowing() then
            floatWindow:show()
            if not hasReportFloatWindowData then
                reportFloatEntranceShowState(FLOAT_WINDON_SHOW);
                hasReportFloatWindowData = true;
            end
        end
    end)
end

local function hideFloatWindow()
    printLog("hideFloatWindow")
    commonlib_runOnUIThread(function()
        if floatWindow:isShowing() then
            floatWindow:hide()
        end
    end)
end

local function onGameOver()
    printLog("onGameOver")
    hideFloatWindow()
    if lastShopMenuPos ~= nil then
        lastShopMenuPos = nil
    end
    hasReportFloatWindowData = false;
end

local function initVariable()
    floatWindow = TopSmallView()
end

local function unInitVariable()
    floatWindow = nil
    lastShopMenuPos = nil
end

local function onFullCoverStateChange(isFullCover)
    printLog("onFullCoverStateChange isFullCover: " .. tostring(isFullCover))
    if isFullCover then
        printLog("界面被全遮挡，不显示悬浮窗")
        hideFloatWindow()
    else
        showFloatWindow()
    end
end

local function onMapModeChange(mapMode)
    printLog("onMapModeChange mapMode: " .. tostring(mapMode))
    if mapMode == LARGE_MAP_LEFT_ZOOMOUT then
        if lastShopMenuPos == SHOP_MENU_POSITION_RIGHT then
            hideFloatWindow()
        else
            showFloatWindow()
        end
    elseif mapMode == LARGE_MAP_RIGHT_ZOOMOUT then
        if lastShopMenuPos == SHOP_MENU_POSITION_LEFT then
            hideFloatWindow()
        else
            showFloatWindow()
        end
    elseif mapMode == LARGE_MAP_HIDE then
        showFloatWindow()
    end
end

local function onChatWindowOpenStateChange(isOpen)
    printLog("onChatWindowOpenStateChange isOpen: " .. tostring(isOpen))
    if isOpen and lastShopMenuPos == SHOP_MENU_POSITION_LEFT then
        hideFloatWindow()
    else
        showFloatWindow()
    end
end

local function onEquipmentChange(shopMenuPosition)
    printLog("onEquipmentChange equipment: " .. tostring(shopMenuPosition))
    if lastShopMenuPos ~= shopMenuPosition then
        moveFloatWindow(shopMenuPosition)
    end
    showFloatWindow()
    lastShopMenuPos = shopMenuPosition
end

local function registerGameEvents()
    commonlib_registerCallback(common_game_events_callbacks.common_game_start, showFloatWindow)
    commonlib_registerCallback(common_game_events_callbacks.common_game_over, onGameOver)
    commonlib_registerCallback(common_game_events_callbacks.common_full_cover, onFullCoverStateChange)
    commonlib_registerCallback(common_game_events_callbacks.common_map_mode, onMapModeChange)
    commonlib_registerCallback(common_game_events_callbacks.common_chat_window, onChatWindowOpenStateChange)
    commonlib_registerCallback(common_game_events_callbacks.common_equipment, onEquipmentChange)
end

local function unregisterGameEvents()
    commonlib_unregisterCallback(common_game_events_callbacks.common_game_start, showFloatWindow)
    commonlib_unregisterCallback(common_game_events_callbacks.common_game_over, onGameOver)
    commonlib_unregisterCallback(common_game_events_callbacks.common_full_cover, onFullCoverStateChange)
    commonlib_unregisterCallback(common_game_events_callbacks.common_map_mode, onMapModeChange)
    commonlib_unregisterCallback(common_game_events_callbacks.common_chat_window, onChatWindowOpenStateChange)
    commonlib_unregisterCallback(common_game_events_callbacks.common_equipment, onEquipmentChange)
end

local function init()
    printLog("init begin")
    registerGameEvents()
    initVariable()
end

local function uninit()
    printLog("uninit")
    unInitVariable()
    unregisterGameEvents()
end

table.insert(common_life_circle_callbacks, { init, uninit })